import datetime
import logging
import os
import time
import warnings

import requests
from alive_progress import alive_bar
from notifypy import Notify

logging.getLogger("playsound").setLevel(logging.CRITICAL)
from playsound import playsound

_NOTIFICATION_ICON = str(os.path.join(os.path.dirname(__file__), "clidoro.png"))


def _playsound(sound: str, sound_dir: str, mode="pomodoro"):
    path = os.path.join(sound_dir, sound + ".mp3")
    if not os.path.exists(path):
        download_assets(
            f"https://kingjuno.github.io/data-server/files/clidoro/assets/{sound}.mp3",
            sound_dir,
        )
    playsound(path, block=False)
    notification = Notify()
    notification.title = "Clidoro"
    notification.icon = _NOTIFICATION_ICON
    notification.message = f"{'Good Job! Take a Break now.' if mode=='pomodoro' else 'Time to start working.'}"
    notification.send()


def _timer(mins, sound="simple-notification", sound_dir=None, mode="pomodoro"):
    try:
        with alive_bar(
            int(mins), spinner="waves", title="Clidoro >", stats="", elapsed=None
        ) as bar:
            for i in range(1, int(1 + 60 * mins)):
                time.sleep(1)
                if i % 60 == 0:
                    bar()
        if sound_dir:
            _playsound(sound, sound_dir, mode)
        return 1
    except KeyboardInterrupt:
        return -1


def date_str_to_datetime(date_str, format="%Y-%m-%d %H:%M:%S.%f"):
    """
    Convert a date string to a datetime object.

    Args:
    - date_str (str): The date string in the specified format.
    - format (str): The format of the date string (default is '%Y-%m-%d %H:%M:%S.%f').

    Returns:
    - datetime.datetime: The datetime object corresponding to the input date string.
    """
    try:
        datetime_obj = datetime.datetime.strptime(date_str, format)
        return datetime_obj
    except ValueError:
        print("Error: Invalid date string or format.")
        return None


def timestamp_to_datetime(timestamp):
    """
    Convert a Unix timestamp to a datetime object.

    Args:
    - timestamp (int): The Unix timestamp to convert.

    Returns:
    - datetime.datetime: The datetime object corresponding to the input timestamp.
    """
    try:
        datetime_obj = datetime.datetime.fromtimestamp(timestamp)
        return datetime_obj
    except ValueError:
        print("Error: Invalid timestamp.")
        return None


def download_assets(link, download_dir="."):
    os.makedirs(download_dir, exist_ok=True)
    filename = link.split("/")[-1]
    response = requests.get(link)
    if response.status_code == 200:
        file_path = os.path.join(download_dir, filename)
        with open(file_path, "wb") as file:
            file.write(response.content)
        print(f"Downloaded: {file_path}")
    else:
        print(f"Failed to download: {link}")



"""
Credits: https://github.com/mkaz/termgraph
Had a small issue with colors while using no_values 
and moreover the repo is maintained 
so copied the codes instead of using it directly
"""
from . import *


from __future__ import print_function
import sys, math, os
import colorama
from typing import Dict, List, Tuple, Union
from .utils import cvt_to_readable

DAYS = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
DELIM = ","
TICK = "▇"
SM_TICK = "▏"

# Commented it out cause I don't know what its purpose is.
# And the code was running just fine without it.
# I am sorry if I am being stupid here.
# try:
#     range = xrange
# except NameError:
#     pass

colorama.init()


class Colors(object):
    """Class representing available color values for graphs."""

    Black = 90
    Red = 91
    Green = 92
    Yellow = 93
    Blue = 94
    Magenta = 95
    Cyan = 96


class Data(object):
    """Class representing the data for the chart."""

    def __init__(
        self,
        data: List,
        labels: List[str],
        categories: List[str] = [],
    ):
        """Initialize data

        :labels: The labels of the data
        :data: The data to graph on the chart
        :categories: The categories of the data
        """

        if len(data) != len(labels):
            raise Exception("The dimensions of the data and labels must be the same")

        self.labels = labels
        self.data = data
        self.categories = categories
        self.dims = self._find_dims(data, labels)

    def _find_dims(self, data, labels, dims=[]) -> Tuple[int]:
        if all([isinstance(data[i], list) for i in range(len(data))]):
            last = None

            for i in range(len(data)):
                curr = self._find_dims(data[i], labels[i], dims + [len(data)])

                if i != 0 and last != curr:
                    raise Exception(
                        f"The inner dimensions of the data are different\nThe dimensions of {data[i - 1]} is different than the dimensions of {data[i]}"
                    )

                last = curr

            return last

        else:
            dims.append(len(data))

        return tuple(dims)

    def find_min(self) -> Union[int, float]:
        """Return the minimum value in sublist of list."""

        return min([min(sublist) for sublist in self.data])

    def find_max(self) -> Union[int, float]:
        """Return the maximum value in sublist of list."""

        return max([max(sublist) for sublist in self.data])

    def find_min_label_length(self) -> int:
        """Return the minimum length for the labels."""

        return min([len(label) for label in self.labels])

    def find_max_label_length(self) -> int:
        """Return the maximum length for the labels."""

        return max([len(label) for label in self.labels])

    def __str__(self):
        """Returns the string representation of the data.
        :returns: The data in a tabular format
        """

        maxlen_labels = max([len(label) for label in self.labels] + [len("Labels")]) + 1

        if len(self.categories) == 0:
            maxlen_data = max([len(str(data)) for data in self.data]) + 1

        else:
            maxlen_categories = max([len(category) for category in self.categories])
            maxlen_data = (
                max(
                    [
                        len(str(self.data[i][j]))
                        for i in range(len(self.data))
                        for j in range(len(self.categories))
                    ]
                )
                + maxlen_categories
                + 4
            )

        output = [
            f"{ ' ' * (maxlen_labels - len('Labels')) }Labels | Data",
            f"{ '-' * (maxlen_labels + 1) }|{ '-' * (maxlen_data + 1) }",
        ]

        for i in range(len(self.data)):
            line = f"{ ' ' * (maxlen_labels - len(self.labels[i])) + self.labels[i] } |"

            if len(self.categories) == 0:
                line += f" {self.data[i]}"

            else:
                for j in range(len(self.categories)):
                    if j == 0:
                        line += f" ({self.categories[j]}) {self.data[i][0]}\n"

                    else:
                        line += f"{ ' ' * maxlen_labels } | ({self.categories[j]}) {self.data[i][j]}"
                        line += (
                            "\n"
                            if j < len(self.categories) - 1
                            else f"\n{ ' ' * maxlen_labels } |"
                        )

            output.append(line)

        return "\n".join(output)

    def __repr__(self):
        return f"Data(data={ self.data if len(str(self.data)) < 25 else str(self.data)[:25] + '...' }, labels={self.labels}, categories={self.categories})"


class Args(object):
    """Class representing the arguments to modify the graph."""

    default = {
        "filename": "-",
        "title": None,
        "width": 50,
        "format": "{:<5.2f}",
        "suffix": "",
        "no_labels": False,
        "no_values": False,
        "space_between": False,
        "colors": None,
        "vertical": False,
        "stacked": False,
        "histogram": False,
        "bins": 5,
        "different_scale": False,
        "calendar": False,
        "start_dt": None,
        "custom_tick": "",
        "delim": "",
        "verbose": False,
        "label_before": False,
    }

    def __init__(self, **kwargs: Dict):
        """Initialize the Args object."""

        self.args = dict(self.default)

        for arg, value in list(kwargs.items()):
            if arg in self.args:
                self.args[arg] = value
            else:
                raise Exception(f"Invalid Argument: {arg}")

    def get_arg(self, arg: str) -> Union[int, str, bool, None]:
        """Returns the value for the argument given.

        :arg: The name of the argument.
        :returns: The value of the argument.

        """

        if arg in self.args:
            return self.args[arg]
        else:
            raise Exception(f"Invalid Argument: {arg}")

    def update_args(self, **kwargs) -> None:
        """Updates the arguments"""

        for arg, value in list(kwargs.items()):
            if arg in self.args:
                self.args[arg] = value
            else:
                raise Exception(f"Invalid Argument: {arg}")


class Chart(object):
    """Class representing a chart"""

    def __init__(self, data: Data, args):
        """Initialize the chart

        :data: The data to be displayed on the chart
        :args: The arguments for the chart

        """

        self.data = data
        self.args = args
        self.normal_data = self._normalize()

    def draw(self) -> None:
        """Draw the chart with the given data"""

        raise NotImplementedError()

    def _print_header(self) -> None:
        title = self.args.get_arg("title")

        if title is not None:
            print(f"# {title}\n")

        if len(self.data.categories) > 0:
            colors = self.args.get_arg("colors")

            for i in range(len(self.data.categories)):
                if colors is not None:
                    sys.stdout.write(
                        "\033[{color_i}m".format(color_i=colors[i])
                    )  # Start to write colorized.
                    sys.stdout.write(f"\033[{colors[i]}m")  # Start to write colorized.

                sys.stdout.write(TICK + " " + self.data.categories[i] + "  ")
                if colors:
                    sys.stdout.write("\033[0m")  # Back to original.

        print("\n\n")

    def _normalize(self) -> List[float]:
        """Normalize the data and return it."""

        # We offset by the minimum if there's a negative.
        data_offset = []
        min_datum = self.data.find_min()

        if min_datum < 0:
            min_datum = abs(min_datum)

            data_offset = [[d + min_datum for d in datum] for datum in self.data.data]

        else:
            data_offset = self.data.data

        max_datum = max([max(sublist) for sublist in data_offset])

        # max_dat / width is the value for a single tick. norm_factor is the
        # inverse of this value
        # If you divide a number to the value of single tick, you will find how
        # many ticks it does contain basically.
        norm_factor = self.args.get_arg("width") / float(max_datum)
        normal_data = [[v * norm_factor for v in datum] for datum in data_offset]

        return normal_data


class HorizontalChart(Chart):
    """Class representing a horizontal chart"""

    def __init__(self, data: Data, args: Args = Args()):
        """Initialize the chart

        :data: The data to be displayed on the chart
        :args: The arguments for the chart

        """

        super().__init__(data, args)

    def print_row(
        self,
        value: Union[int, float],
        num_blocks: Union[int, float],
        val_min: Union[int, float],
        color: int,
        label: bool = False,
        tail: bool = False,
    ) -> None:
        """A method to print a row for a horizontal graphs.
        i.e:
        1: ▇▇ 2
        2: ▇▇▇ 3
        3: ▇▇▇▇ 4
        """
        doprint = self.args.get_arg("label_before") and not self.args.get_arg(
            "vertical"
        )

        sys.stdout.write("\033[0m")  # no color

        if value == 0.0:
            sys.stdout.write(f"\033[{Colors.Black}m")  # dark gray

        if doprint:
            print(label, tail, " ", end="")

        if (num_blocks < 1 and (value > val_min or value > 0)) or (
            self.args.get_arg("label_before") and value == 0.0
        ):
            # Print something if it's not the smallest
            # and the normal value is less than one.
            sys.stdout.write(SM_TICK)

        else:
            if color:
                sys.stdout.write(f"\033[{color}m")  # Start to write colorized.

            for _ in range(num_blocks):
                sys.stdout.write(TICK)

        if color:
            sys.stdout.write("\033[0m")  # Back to original.

        if doprint:
            print()


class BarChart(HorizontalChart):
    """Class representing a bar chart"""

    def __init__(self, data: Data, args: Args = Args()):
        """Initialize the bar chart

        :data: The data to be displayed on the chart
        :args: The arguments for the chart

        """

        super().__init__(data, args)

    def draw(self) -> None:
        """Draws the chart"""
        self._print_header()

        colors = (
            self.args.get_arg("colors")
            if self.args.get_arg("colors") != None
            else [None] * self.data.dims[1]
        )

        val_min = self.data.find_min()

        for i in range(len(self.data.labels)):
            if self.args.get_arg("no_labels"):
                # Hide the labels.
                label = ""
            else:
                if self.args.get_arg("label_before"):
                    fmt = "{:<{x}}"
                else:
                    fmt = "{:<{x}}: "

                label = fmt.format(
                    self.data.labels[i], x=self.data.find_max_label_length()
                )

            values = self.data.data[i]
            num_blocks = self.normal_data[i]

            if self.args.get_arg("space_between") and i != 0:
                print()

            for j in range(len(values)):
                # In Multiple series graph 1st category has label at the beginning,
                # whereas the rest categories have only spaces.
                if j > 0:
                    len_label = len(label)
                    label = " " * len_label

                if self.args.get_arg("label_before"):
                    fmt = "{}{}{}"

                else:
                    fmt = " {}{}{}"

                if self.args.get_arg("no_values"):
                    tail = self.args.get_arg("suffix")

                else:
                    val, deg = cvt_to_readable(values[j])
                    tail = fmt.format(
                        self.args.get_arg("format").format(val),
                        deg,
                        self.args.get_arg("suffix"),
                    )

                if colors:
                    color = colors[j]

                else:
                    color = None

                if not self.args.get_arg("label_before") and not self.args.get_arg(
                    "vertical"
                ):
                    print(label, end="")

                self.print_row(
                    values[j],
                    int(num_blocks[j]),
                    val_min,
                    color,
                    label,
                    tail,
                )

                if not self.args.get_arg("label_before") and not self.args.get_arg(
                    "vertical"
                ):
                    print(tail)



"""This module allows drawing basic graphs in the terminal."""

# termgraph.py - draw basic graphs on terminal
# https://github.com/mkaz/termgraph

from __future__ import print_function, annotations
from typing import List, Tuple, Dict
import argparse
import sys
import math
from datetime import datetime, timedelta
from itertools import zip_longest
from colorama import init  # type: ignore
import os
import re

VERSION = "0.5.3"

init()

# ANSI escape SGR Parameters color codes
AVAILABLE_COLORS = {
    "red": 91,
    "blue": 94,
    "green": 92,
    "magenta": 95,
    "yellow": 93,
    "black": 90,
    "cyan": 96,
}

DAYS = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
UNITS = ["", "K", "M", "B", "T"]
DELIM = ","
TICK = "▇"
SM_TICK = "▏"

try:
    range = xrange  # type: ignore
except NameError:
    pass


def init_args() -> Dict:
    """Parse and return the arguments."""
    parser = argparse.ArgumentParser(description="draw basic graphs on terminal")
    parser.add_argument(
        "filename",
        nargs="?",
        default="-",
        help="data file name (comma or space separated). Defaults to stdin.",
    )
    parser.add_argument("--title", help="Title of graph")
    parser.add_argument(
        "--width", type=int, default=50, help="width of graph in characters default:50"
    )
    parser.add_argument("--format", default="{:<5.2f}", help="format specifier to use.")
    parser.add_argument(
        "--suffix", default="", help="string to add as a suffix to all data points."
    )
    parser.add_argument(
        "--no-labels", action="store_true", help="Do not print the label column"
    )
    parser.add_argument(
        "--no-values", action="store_true", help="Do not print the values at end"
    )
    parser.add_argument(
        "--space-between",
        action="store_true",
        help="Print a new line after every field",
    )
    parser.add_argument("--color", nargs="*", help="Graph bar color( s )")
    parser.add_argument("--vertical", action="store_true", help="Vertical graph")
    parser.add_argument("--stacked", action="store_true", help="Stacked bar graph")
    parser.add_argument("--histogram", action="store_true", help="Histogram")
    parser.add_argument("--bins", default=5, type=int, help="Bins of Histogram")
    parser.add_argument(
        "--different-scale",
        action="store_true",
        help="Categories have different scales.",
    )
    parser.add_argument(
        "--calendar", action="store_true", help="Calendar Heatmap chart"
    )
    parser.add_argument("--start-dt", help="Start date for Calendar chart")
    parser.add_argument(
        "--custom-tick", default="", help="Custom tick mark, emoji approved"
    )
    parser.add_argument(
        "--delim", default="", help="Custom delimiter, default , or space"
    )
    parser.add_argument(
        "--verbose", action="store_true", help="Verbose output, helpful for debugging"
    )
    parser.add_argument(
        "--label-before",
        action="store_true",
        default=False,
        help="Display the values before the bars",
    )
    parser.add_argument(
        "--version", action="store_true", help="Display version and exit"
    )
    if len(sys.argv) == 1:
        if sys.stdin.isatty():
            parser.print_usage()
            sys.exit(2)

    args = vars(parser.parse_args())

    if args["custom_tick"] != "":
        global TICK, SM_TICK
        TICK = args["custom_tick"]
        SM_TICK = ""

    if args["delim"] != "":
        global DELIM
        DELIM = args["delim"]

    return args


def main():
    """Main function."""
    args = init_args()

    if args["version"]:
        print("termgraph v{}".format(VERSION))
        sys.exit()

    _, labels, data, colors = read_data(args)
    try:
        if args["calendar"]:
            calendar_heatmap(data, labels, args)
        else:
            chart(colors, data, args, labels)
    except BrokenPipeError:
        pass


def find_min(data: List):
    """Return the minimum value in sublist of list."""
    return min([min(sublist) for sublist in data])


def find_max(data: List):
    """Return the maximum value in sublist of list."""
    return max([max(sublist) for sublist in data])


def normalize(data: List, width: int) -> List:
    """Normalize the data and return it."""

    # We offset by the minimum if there's a negative.
    data_offset = []
    min_datum = find_min(data)
    if min_datum < 0:
        min_datum = abs(min_datum)
        for datum in data:
            data_offset.append([d + min_datum for d in datum])
    else:
        data_offset = data
    min_datum = find_min(data_offset)
    max_datum = find_max(data_offset)

    if min_datum == max_datum:
        return data_offset

    # max_dat / width is the value for a single tick. norm_factor is the
    # inverse of this value
    # If you divide a number to the value of single tick, you will find how
    # many ticks it does contain basically.
    norm_factor = width / float(max_datum)
    normal_data = []
    for datum in data_offset:
        normal_data.append([v * norm_factor for v in datum])

    return normal_data


def find_max_label_length(labels: List) -> int:
    """Return the maximum length for the labels."""
    return max([len(label) for label in labels])


def cvt_to_readable(num):
    """Return the number in a human readable format

    Eg:
    125000 -> 125.0K
    12550 -> 12.55K
    19561100 -> 19.561M
    """

    if num != 0:
        neg = num < 0
        num = abs(num)

        # Find the degree of the number like if it is in thousands or millions, etc.
        index = math.floor(math.log(num) / math.log(1000))

        # Converts the number to the human readable format and returns it.
        newNum = round(num / (1000**index), 3)
        newNum *= -1 if neg else 1
        degree = UNITS[index]

    else:
        newNum = 0
        degree = UNITS[0]

    return (newNum, degree)


def hist_rows(data: List, args: Dict, colors: List):
    """Prepare the Histgram graph.
    Each row is printed through the print_row function."""

    val_min = find_min(data)
    val_max = find_max(data)

    # Calculate borders
    class_min = math.floor(val_min)
    class_max = math.ceil(val_max)
    class_range = class_max - class_min
    class_width = class_range / args["bins"]

    border = class_min
    borders = []
    max_len = len(str(border))

    for b in range(args["bins"] + 1):
        borders.append(border)
        len_border = len(str(border))
        if len_border > max_len:
            max_len = len_border
        border += class_width
        border = round(border, 1)

    # Count num of data via border
    count_list = []

    for start, end in zip(borders[:-1], borders[1:]):
        count = 0
        #        for d in [d]
        for v in [row[0] for row in data]:
            if start <= v < end:
                count += 1

        count_list.append([count])

    normal_counts = normalize(count_list, args["width"])

    for i, border in enumerate(zip(borders[:-1], borders[1:])):
        if colors:
            color = colors[0]
        else:
            color = None

        if not args.get("no_labels"):
            print("{:{x}} – {:{x}}: ".format(border[0], border[1], x=max_len), end="")

        num_blocks = normal_counts[i]

        yield (count_list[i][0], int(num_blocks[0]), 0, color)

        if args.get("no_values"):
            tail = ""
        else:
            tail = " {}{}".format(
                args["format"].format(count_list[i][0]), args["suffix"]
            )
        print(tail)


def horiz_rows(
    labels: List,
    data: List,
    normal_dat: List,
    args: Dict,
    colors: List,
    doprint: bool = True,
):
    """Prepare the horizontal graph.
    Each row is printed through the print_row function."""
    val_min = find_min(data)

    for i in range(len(labels)):
        if args["no_labels"]:
            # Hide the labels.
            label = ""
        else:
            if args.get("label_before"):
                fmt = "{:<{x}}"
            else:
                fmt = "{:<{x}}: "
            label = fmt.format(labels[i], x=find_max_label_length(labels))

        values = data[i]
        num_blocks = normal_dat[i]

        if args.get("space_between") and i != 0:
            print()

        for j in range(len(values)):
            # In Multiple series graph 1st category has label at the beginning,
            # whereas the rest categories have only spaces.
            if j > 0:
                len_label = len(label)
                label = " " * len_label
            if args.get("label_before"):
                fmt = "{}{}{}"
            else:
                fmt = " {}{}{}"

            if args["no_values"]:
                tail = args["suffix"]
            else:
                val, deg = cvt_to_readable(values[j])
                tail = fmt.format(args["format"].format(val), deg, args["suffix"])

            if colors:
                color = colors[j]
            else:
                color = None

            if not args.get("label_before") and not args.get("vertical"):
                print(label, end="")

            yield (
                values[j],
                int(num_blocks[j]),
                val_min,
                color,
                label,
                tail,
                args.get("label_before") and not args.get("vertical"),
            )

            if not args.get("label_before") and not args.get("vertical"):
                print(tail)


# Prints a row of the horizontal graph.
def print_row(
    value,
    num_blocks: int,
    val_min: int,
    color: bool,
    label: bool = False,
    tail: bool = False,
    doprint: bool = False,
):
    """A method to print a row for a horizontal graphs.
    i.e:
    1: ▇▇ 2
    2: ▇▇▇ 3
    3: ▇▇▇▇ 4
    """
    sys.stdout.write("\033[0m")  # no color
    if value == 0.0:
        sys.stdout.write("\033[90m")  # dark gray

    if doprint:
        print(label, tail, " ", end="")

    if (num_blocks < 1 and (value > val_min or value > 0)) or (
        doprint and value == 0.0
    ):
        # Print something if it's not the smallest
        # and the normal value is less than one.
        sys.stdout.write(SM_TICK)
    else:
        if color:
            sys.stdout.write(f"\033[{color}m")  # Start to write colorized.
        for _ in range(num_blocks):
            sys.stdout.write(TICK)

    if color:
        sys.stdout.write("\033[0m")  # Back to original.

    if doprint:
        print()


def stacked_graph(
    labels: List,
    data: List,
    normal_data: List,
    len_categories: int,
    args: Dict,
    colors: List,
):
    """Prepare the horizontal stacked graph.
    Each row is printed through the print_row function."""
    val_min = find_min(data)

    for i in range(len(labels)):
        if args["no_labels"]:
            # Hide the labels.
            label = ""
        else:
            label = "{:<{x}}: ".format(labels[i], x=find_max_label_length(labels))

        if args.get("space_between") and i != 0:
            print()

        print(label, end="")

        values = data[i]
        num_blocks = normal_data[i]

        for j in range(len(values)):
            print_row(values[j], int(num_blocks[j]), val_min, colors[j])

        tail = " {}{}".format(args["format"].format(sum(values)), args["suffix"])
        print(tail)


# FIXME: globals for vertical, not ideal
value_list, zipped_list, vertical_list, maxi = [], [], [], 0


def vertically(value, num_blocks: int, val_min: int, color: bool, args: Dict) -> List:
    """Prepare the vertical graph.
    The whole graph is printed through the print_vertical function."""
    global maxi, value_list

    value_list.append(str(value))

    # In case the number of blocks at the end of the normalization is less
    # than the default number, use the maxi variable to escape.
    if maxi < num_blocks:
        maxi = num_blocks

    if num_blocks > 0:
        vertical_list.append((TICK * num_blocks))
    else:
        vertical_list.append(SM_TICK)

    # Zip_longest method in order to turn them vertically.
    for row in zip_longest(*vertical_list, fillvalue=" "):
        zipped_list.append(row)

    counter, result_list = 0, []

    # Combined with the maxi variable, escapes the appending method at
    # the correct point or the default one (width).
    for i in reversed(zipped_list):
        result_list.append(i)
        counter += 1

        if maxi == args["width"]:
            if counter == (args["width"]):
                break
        else:
            if counter == maxi:
                break

    # Return a list of rows which will be used to print the result vertically.
    return result_list


def print_vertical(vertical_rows: List, labels: List, color: bool, args: Dict) -> None:
    """Print the whole vertical graph."""
    if color:
        sys.stdout.write(
            "\033[{color}m".format(color=color)
        )  # Start to write colorized.

    for row in vertical_rows:
        print(*row)

    sys.stdout.write("\033[0m")  # End of printing colored

    if not args["no_values"]:
        print("-" * len(row) + "Values" + "-" * len(row))
        for value in zip_longest(*value_list, fillvalue=" "):
            print("  ".join(value))

    if not args["no_labels"]:
        print("-" * len(row) + "Labels" + "-" * len(row))
        # Print Labels
        for label in zip_longest(*labels, fillvalue=""):
            print("  ".join(label))


def chart(colors: List, data: List, args: Dict, labels: List) -> None:
    """Handle the normalization of data and the printing of the graph."""
    len_categories = len(data[0])
    if len_categories > 1:
        # Stacked graph
        if args["stacked"]:
            normal_dat = normalize(data, args["width"])
            stacked_graph(labels, data, normal_dat, len_categories, args, colors)
            return

        if not colors:
            colors = [None] * len_categories

        # Multiple series graph with different scales
        # Normalization per category
        if args["different_scale"]:
            for i in range(len_categories):
                cat_data = []
                for dat in data:
                    cat_data.append([dat[i]])

                # Normalize data, handle negatives.
                normal_cat_data = normalize(cat_data, args["width"])

                # Generate data for a row.
                for row in horiz_rows(
                    labels, cat_data, normal_cat_data, args, [colors[i]]
                ):
                    # Print the row
                    if args["vertical"]:
                        # FIXME: passing args is getting complex
                        vertic = vertically(row[0], row[1], row[2], row[3], args=args)
                    else:
                        print_row(*row)
                        print("\n")

                # The above gathers data for vertical and does not print
                # the final print happens at once here
                if args["vertical"]:
                    print_vertical(vertic, labels, colors[i], args)

                print()
                value_list.clear(), zipped_list.clear(), vertical_list.clear()
            return

    if args["histogram"]:
        if args["vertical"]:
            print(">> Error: Vertical graph for Histogram" " is not supported yet.")
            sys.exit(1)

        for row in hist_rows(data, args, colors):
            print_row(*row)
            print()
        return

    # One category/Multiple series graph with same scale
    # All-together normalization
    if not args["stacked"]:
        normal_dat = normalize(data, args["width"])
        sys.stdout.write("\033[0m")  # no color
        for row in horiz_rows(labels, data, normal_dat, args, colors):
            if not args["vertical"]:
                print_row(*row)
            else:
                # FIXME: passing args is getting complex
                vertic = vertically(row[0], row[1], row[2], row[3], args=args)

        if args["vertical"] and len_categories == 1:
            if colors:
                color = colors[0]
            else:
                color = None

            print_vertical(vertic, labels, color, args)

        print()


def check_data(labels: List, data: List, args: Dict) -> List:
    """Check that all data were inserted correctly. Return the colors."""
    len_categories = len(data[0])

    # Check that there are data for all labels.
    if len(labels) != len(data):
        print(">> Error: Label and data array sizes don't match")
        sys.exit(1)

    # Check that there are data for all categories per label.
    for dat in data:
        if len(dat) != len_categories:
            print(">> Error: There are missing values")
            sys.exit(1)

    colors = []

    # If user inserts colors, they should be as many as the categories.
    if args["color"] is not None:
        # Decompose arguments for Windows
        if os.name == "nt":
            colorargs = re.findall(r"[a-z]+", args["color"][0])
            if len(colorargs) != len_categories:
                print(">> Error: Color and category array sizes don't match")
            for color in colorargs:
                if color not in AVAILABLE_COLORS:
                    print(
                        ">> Error: invalid color. choose from 'red', 'blue', 'green', 'magenta', 'yellow', 'black', 'cyan'"
                    )
                    sys.exit(2)
        else:
            if len(args["color"]) != len_categories:
                print(">> Error: Color and category array sizes don't match")
            for color in args["color"]:
                if color not in AVAILABLE_COLORS:
                    print(
                        ">> Error: invalid color. choose from 'red', 'blue', 'green', 'magenta', 'yellow', 'black', 'cyan'"
                    )
                    sys.exit(2)

        if os.name == "nt":
            for color in colorargs:
                colors.append(AVAILABLE_COLORS.get(color))
        else:
            for color in args["color"]:
                colors.append(AVAILABLE_COLORS.get(color))

    # Vertical graph for multiple series of same scale is not supported yet.
    if args["vertical"] and len_categories > 1 and not args["different_scale"]:
        print(
            ">> Error: Vertical graph for multiple series of same "
            "scale is not supported yet."
        )
        sys.exit(1)

    # If user hasn't inserted colors, pick the first n colors
    # from the dict (n = number of categories).
    if args["stacked"] and not colors:
        colors = [v for v in list(AVAILABLE_COLORS.values())[:len_categories]]

    return colors


def print_categories(categories: List, colors: List) -> None:
    """Print a tick and the category's name for each category above
    the graph."""
    for i in range(len(categories)):
        if colors:
            sys.stdout.write(
                "\033[{color_i}m".format(color_i=colors[i])
            )  # Start to write colorized.
            sys.stdout.write(f"\033[{colors[i]}m")  # Start to write colorized.

        sys.stdout.write(TICK + " " + categories[i] + "  ")
        if colors:
            sys.stdout.write("\033[0m")  # Back to original.

    print("\n\n")


def read_data(args: Dict) -> Tuple[List, List, List, List]:
    """Read data from a file or stdin and returns it.

    Filename includes (categories), labels and data.
    We append categories and labels to lists.
    Data are inserted to a list of lists due to the categories.

    i.e.
    labels = ['2001', '2002', '2003', ...]
    categories = ['boys', 'girls']
    data = [ [20.4, 40.5], [30.7, 100.0], ...]"""

    filename = args["filename"]
    stdin = filename == "-"

    if args["verbose"]:
        print(">> Reading data from {src}".format(src=("stdin" if stdin else filename)))

    print("")
    if args["title"]:
        print("# " + args["title"] + "\n")

    categories, labels, data, colors = ([] for _ in range(4))

    f = None

    try:
        f = sys.stdin if stdin else open(filename, "r")
        for line in f:
            line = line.strip()
            if line:
                if not line.startswith("#"):
                    if line.find(DELIM) > 0:
                        cols = line.split(DELIM)
                    else:
                        cols = line.split()

                    # Line contains categories.
                    if line.startswith("@"):
                        cols[0] = cols[0].replace("@ ", "")
                        categories = cols

                    # Line contains label and values.
                    else:
                        labels.append(cols[0].strip())
                        data_points = []
                        for i in range(1, len(cols)):
                            data_points.append(float(cols[i].strip()))

                        data.append(data_points)
    except FileNotFoundError:
        print(
            ">> Error: The specified file [{fname}] does not exist.".format(
                fname=filename
            )
        )
        sys.exit()
    except IOError:
        print("An IOError has occurred!")
        sys.exit()
    finally:
        if f is not None:
            f.close()

    # Check that all data are valid. (i.e. There are no missing values.)
    colors = check_data(labels, data, args)
    if categories:
        # Print categories' names above the graph.
        print_categories(categories, colors)

    return categories, labels, data, colors


def calendar_heatmap(data: Dict, labels: List, args: Dict) -> None:
    """Print a calendar heatmap."""
    if args["color"]:
        colornum = AVAILABLE_COLORS.get(args["color"][0])
    else:
        colornum = AVAILABLE_COLORS.get("blue")

    dt_dict = {}
    for i in range(len(labels)):
        dt_dict[labels[i]] = data[i][0]

    # get max value
    max_val = float(max(data)[0])

    tick_1 = "░"
    tick_2 = "▒"
    tick_3 = "▓"
    tick_4 = "█"

    if args["custom_tick"]:
        tick_1 = tick_2 = tick_3 = tick_4 = args["custom_tick"]

    # check if start day set, otherwise use one year ago
    if args["start_dt"]:
        start_dt = datetime.strptime(args["start_dt"], "%Y-%m-%d")
    else:
        start = datetime.now()
        start_dt = datetime(year=start.year - 1, month=start.month, day=start.day)

    # modify start date to be a Monday, subtract weekday() from day
    start_dt = start_dt - timedelta(start_dt.weekday())

    # TODO: legend doesn't line up properly for all start dates/data
    # top legend for months
    sys.stdout.write("     ")
    for month in range(13):
        month_dt = datetime(
            year=start_dt.year, month=start_dt.month, day=1
        ) + timedelta(days=month * 31)
        sys.stdout.write(month_dt.strftime("%b") + " ")
        if args["custom_tick"]:  # assume custom tick is emoji which is one wider
            sys.stdout.write(" ")

    sys.stdout.write("\n")

    for day in range(7):
        sys.stdout.write(DAYS[day] + ": ")
        for week in range(53):
            day_ = start_dt + timedelta(days=day + week * 7)
            day_str = day_.strftime("%Y-%m-%d")

            if day_str in dt_dict:
                if dt_dict[day_str] > max_val * 0.75:
                    tick = tick_4
                elif dt_dict[day_str] > max_val * 0.50:
                    tick = tick_3
                elif dt_dict[day_str] > max_val * 0.25:
                    tick = tick_2
                # show nothing if value is zero
                elif dt_dict[day_str] == 0.0:
                    tick = " "
                # show values for less than 0.25
                else:
                    tick = tick_1
            else:
                tick = " "

            if colornum:
                sys.stdout.write("\033[{colornum}m".format(colornum=colornum))

            sys.stdout.write(tick)
            if colornum:
                sys.stdout.write("\033[0m")

        sys.stdout.write("\n")



import math

UNITS = ["", "K", "M", "B", "T"]


def cvt_to_readable(num):
    """Return the number in a human readable format

    Eg:
    125000 -> 125.0K
    12550 -> 12.55K
    19561100 -> 19.561M
    """
    if num==0:
        return (0, "")
    # Find the degree of the number like if it is in thousands or millions, etc.
    index = int(math.log(num) / math.log(1000))

    # Converts the number to the human readable format and returns it.
    newNum = round(num / (1000 ** index), 3)
    degree = UNITS[index]

    return (newNum, degree)


from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = "\n" + f.read()

MAJOR_VERSION = "0"
MINOR_VERSION = "1"
MICRO_VERSION = "2"
VERSION = "{}.{}.{}".format(MAJOR_VERSION, MINOR_VERSION, MICRO_VERSION)


setup(
    name="clidoro",
    packages=find_packages(exclude=["docs", "tests*", "examples"]),
    version=VERSION,
    license="MIT",
    zip_safe=False,
    platforms="Linux",
    description="clidoro: pomodoro in your cli",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kingjuno/clidoro",
    entry_points={"console_scripts": ["clidoro = clidoro.clidoro:main"]},
    keywords=["Productivity", "Pomodoro"],
    install_requires=[
        "playsound",
        "alive-progress",
        "simple-term-menu",
        "requests",
        "colorama",
        "notify-py"
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Customer Service",
        "Intended Audience :: System Administrators",
        "Operating System :: Unix",
        # TODO: Add Windows support
        # TODO: Add Mac support
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Software Distribution",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
    ],
    package_data={
        "clidoro": ["*.png"],
    },
)



import io
import sqlite3
from contextlib import redirect_stdout
from datetime import date, datetime

from clidoro._utils import date_str_to_datetime
from clidoro.termgraph.module import Args, BarChart, Colors, Data

GREEN_COLOR = "\033[92m"
RED_COLOR = "\033[91m"
RESET_COLOR = "\033[0m"


def history_all(cache_dir):
    try:
        conn = sqlite3.connect(f"{cache_dir}/clidoro.sqlite")
        cursor = conn.cursor()
        cursor.execute("SELECT date, amount FROM random_data")
        rows = cursor.fetchall()
        conn.close()
        dates = [list(row) for row in rows]
        if len(dates) == 0: raise
    except:
        return f"{RED_COLOR}No data found.{RESET_COLOR}\n"

    todays_date = date.today()
    todays_times = [
        row[1]
        for row in dates
        if datetime.strptime(row[0], "%Y-%m-%d %H:%M:%S.%f").date() == todays_date
    ]
    times = [time[1] for time in dates]
    stats = (
        f"{GREEN_COLOR}Pomodoros Today: {RED_COLOR}{len(todays_times)}{RESET_COLOR}\n"
        + f"{GREEN_COLOR}Hours Today: {RED_COLOR}{sum(todays_times)/60:.2f}{RESET_COLOR}\n"
    )

    stats += (
        f"\n{GREEN_COLOR}Pomodoros Total: {RED_COLOR}{len(times)}{RESET_COLOR}\n"
        f"{GREEN_COLOR}Hours Total: {RED_COLOR}{sum(times)/60:.2f}{RESET_COLOR}\n"
    )
    return stats


def history(cache_dir):
    try:
        conn = sqlite3.connect(f"{cache_dir}/clidoro.sqlite")
        cursor = conn.cursor()
        cursor.execute("SELECT date, amount FROM random_data")
        rows = cursor.fetchall()
        conn.close()
        dates = [list(row) for row in rows]
        if len(dates) == 0: raise
    except:
        return f"{RED_COLOR}No data found.{RESET_COLOR}\n"
    
    timers = set([timer[-1] for timer in dates])
    if len(dates) == 0:
        return f"{RED_COLOR}No data found.{RESET_COLOR}\n"
    current_week = datetime.now().isocalendar()[1]
    weekly_stats = {
        "Monday": {timer: 0 for timer in timers},
        "Tuesday": {timer: 0 for timer in timers},
        "Wednesday": {timer: 0 for timer in timers},
        "Thursday": {timer: 0 for timer in timers},
        "Friday": {timer: 0 for timer in timers},
        "Saturday": {timer: 0 for timer in timers},
        "Sunday": {timer: 0 for timer in timers},
    }
    for date, timer in dates:
        date = date_str_to_datetime(date)
        if date.isocalendar()[1] == current_week:
            weekday = date.strftime("%A")
            weekly_stats[weekday][timer] += 1

    week_data = [[sum([i[j] for j in i])] for i in weekly_stats.values()]
    data = Data(week_data, list(weekly_stats.keys()), ["Pomodoros by Week Day"])
    chart = BarChart(
        data,
        Args(
            title=None,
            colors=[Colors.Red, Colors.Magenta, Colors.Blue],
            format="{:0.0f}",
        ),
    )
    f = io.StringIO()
    with redirect_stdout(f):
        chart.draw()
    return f.getvalue()



import time
from pathlib import Path

from simple_term_menu import TerminalMenu

from clidoro._utils import _timer, timestamp_to_datetime
from clidoro.chart import history, history_all
from clidoro.data import clear_db, save_to_db

TITLE = """        _   _     _                 
       | | (_)   | |                
   ___ | |  _  __| | ___  _ __ ___  
  / __|| | | |/ _` |/ _ \| '__/ _ \ 
 | (__ | | | | (_| | (_) | | | (_) |
  \___||_| |_|\__,_|\___/|_|  \___/ 
  
  author:kingjuno
"""
CACHE_DIR = Path.home() / ".cache" / "clidoro"


def timer(menu, _save=False):
    """
    pomodoro
    index
    0-5: ["20 mins", "25 mins", "30 mins", "45 mins", "1  hour", "back"]
    prograssable terminal time option
    """
    times = [int(item.split(" ")[0]) for item in menu._menu_entries[:-1]]
    start_time = timestamp_to_datetime(time.time())
    while True:
        menu_entry_index = menu.show()
        if menu_entry_index == len(menu._menu_entries) - 1:
            break
        else:
            _time = times[menu_entry_index]
            stats = _timer(
                0.01, "simple-notification", CACHE_DIR, "pomodoro" if _save else "break"
            )
        if _save and stats > 0:
            save_to_db([[start_time, _time]], CACHE_DIR)


def history_menu(menu):
    while True:
        menu_entry_index = menu.show()
        if menu_entry_index == len(menu._menu_entries) - 1:
            break
        elif menu_entry_index == 0:
            _history = history(CACHE_DIR).replace("\n\n", "\n")
        elif menu_entry_index == 1:
            _history = history_all(CACHE_DIR)
        elif menu_entry_index == 2:
            delete_menu = TerminalMenu(
                ["accept", "exit"],
                accept_keys=("enter", "alt-d"),
                menu_highlight_style=("bg_black", "fg_green"),
            )
            delete_index = delete_menu.show()
            if delete_index == 0:
                clear_db(CACHE_DIR)
            return
        print(TITLE, _history, sep="\n")
        terminal_menu = TerminalMenu(
            ["exit"],
            accept_keys=("enter", "alt-d"),
            menu_highlight_style=("bg_black", "fg_green"),
        )
        terminal_menu.show()


def main():
    cli_options = ["start", "break", "history", "exit"]
    pomodoro_options = ["20 mins", "25 mins", "30 mins", "45 mins", "60 mins", "back"]
    break_options = ["5 mins", "10 mins", "15 mins", "30 mins", "back"]
    history_options = ["Pomodoros by weekdays", "Stats", "clear DB", "exit"]
    terminal_menu = TerminalMenu(
        cli_options,
        title=TITLE,
        accept_keys=("enter", "alt-d"),
        clear_screen=True,
        menu_highlight_style=("bg_black", "fg_green"),
    )
    pomodoro_menu = TerminalMenu(
        pomodoro_options,
        title=TITLE,
        accept_keys=("enter", "alt-d"),
        clear_screen=True,
        menu_highlight_style=("bg_black", "fg_green"),
    )
    break_menu = TerminalMenu(
        break_options,
        title=TITLE,
        accept_keys=("enter", "alt-d"),
        clear_screen=True,
        menu_highlight_style=("bg_black", "fg_green"),
    )
    hist_menu = TerminalMenu(
        history_options,
        title=TITLE,
        accept_keys=("enter", "alt-d"),
        clear_screen=True,
        menu_highlight_style=("bg_black", "fg_green"),
    )
    while not False:
        menu_entry_index = terminal_menu.show()
        if menu_entry_index == 0:
            timer(pomodoro_menu, _save=True)
        elif menu_entry_index == 1:
            timer(break_menu)
        elif menu_entry_index == 2:
            history_menu(hist_menu)
        if menu_entry_index == 3:
            break


if __name__ == "__main__":
    main()



import os
import sqlite3


def save_to_db(dates, cache_dir):
    conn = sqlite3.connect(f"{cache_dir}/clidoro.sqlite")
    cursor = conn.cursor()

    cursor.execute(
        """
        CREATE TABLE IF NOT EXISTS random_data (
            id INTEGER PRIMARY KEY,
            date TEXT,
            amount INTEGER
        )
    """
    )
    cursor.executemany("INSERT INTO random_data (date, amount) VALUES (?, ?)", dates)
    conn.commit()
    conn.close()


def clear_db(cache_dir):
    db_path = os.path.join(cache_dir, "clidoro.sqlite")
    try:
        if os.path.exists(db_path):
            os.remove(db_path)
    except:
        ...
